#!/usr/bin/env python
#
# Author: Shreekar Patel 03/28/2014 (Shreekar.Patel@uth.tmc.edu)
# Copyright (c) 2014 The University of Texas - Houston Medical School
#
# This software is issued under a joint BSD/GNU license. You may use the
# source code in this file under either license. However, note that the
# complete EMAN2 and SPARX software packages have some GPL dependencies,
# so you are responsible for compliance with the licenses of these packages
# if you opt to use BSD licensing. The warranty disclaimer below holds
# in either instance.
#
# This complete copyright notice must be included in any revised version of the
# source code. Additional authorship citations may be added, but existing
# author citations must be preserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
import os, sys
import json
from optparse import OptionParser

from EMAN2 import EMData, Region, Util
from utilities import read_text_row, set_ctf
from filter import filt_gaussh


def read_coordinates(location):
	"""
	Read coordinates from .json files. Each micrograph outputs separate json file.
	Usually generated by executing:
		$ e2boxer.py --write_dbbox mic*.hdf --format=json
	"""
	output = {}
	for _f in os.listdir(location):
		if _f.endswith('.json'):
			data = json.loads(open(os.path.join(location, _f), 'r').read())
			for mic_name, coordinates in data.items():
				output[str(mic_name)] = coordinates
	return output


def window(data):
	"""
	Using coordinates window particles, and add ctf information to it.
	"""
	for k, info in data.items():
		print 'Processing {0}'.format(k)
		box_size = data[k]['box_size']
		pixel_ratio = float(data[k]['input_pixel'])/float(data[k]['output_pixel'])
		img = EMData()
		img.read_image(k)
		img_filt = filt_gaussh(img, pixel_ratio/box_size)


		if pixel_ratio != 1.0:
			print "Generating downsampled image\n"
			sb = Util.sincBlackman(15, .5 * pixel_ratio,1999) # 1999 taken directly from util_sparx.h
			img_filt = img_filt.downsample(sb, pixel_ratio)
			box_size = box_size / pixel_ratio

		output_file_name = 'out_' + os.path.basename(k)
		clip = EMData()
		for i, (x, y) in enumerate(data[k]['coordinates']):
			reg = Region((x * pixel_ratio)-box_size//2, (y * pixel_ratio)-box_size//2, box_size, box_size)
			clip = img_filt.get_clip(reg)
			clip.write_image(output_file_name, i)
		# Set ctf
		set_ctf(clip, data[k]['ctf'])
		print 'Windowed prticles for {0} -> {1}'.format(k, output_file_name)


def main():
	parser = OptionParser()
	parser.add_option('-c', '--coord', dest='coord', help='location where coordinates are located')
	parser.add_option('-f', '--ctf', dest='ctf_file', help='ctf information file')
	parser.add_option('-m', '--mic_dir', dest='mic_dir', help='micrograph location')
	parser.add_option('-i', '--input_pixel', dest='input_pixel', help='input pixel size', default=1)
	parser.add_option('-o', '--output_pixel', dest='output_pixel', help='output pixel size', default=1)
	(options, args) = parser.parse_args()

	if options.coord and os.path.exists(options.coord):
		coordinates = read_coordinates(options.coord)
	else:
		parser.error('Invalid location for coordinates file')

	if options.ctf_file and os.path.exists(options.ctf_file):
		ctf = read_text_row(options.ctf_file)
		# Unfortunately, you get 'list of list' that also contains file name.
		ctf_dir = {}
		for x in ctf:
			ctf_dir[os.path.basename(x[-1])] = x[:-1]
	else:
		parser.error('Invalid file path for CTF information')

	if not options.mic_dir or not os.path.exists(options.mic_dir):
		parser.error('Invalid location for micrographs.')

	# Process coordinates and ctf information into one data structure for easy access
	if len(coordinates) and len(ctf) and len(coordinates) == len(ctf):
		# I think we have same number of micrograph
		data = {}
		for mic_name, coordinates in coordinates.items():
			# Add coordinates
			mic_path = os.path.join(options.mic_dir, mic_name)
			data[mic_path] = coordinates

			# Add CTF information
			data[mic_path]['ctf'] = ctf_dir[mic_name]

			# Add input output pixel size
			data[mic_path]['input_pixel'], data[mic_path]['output_pixel'] = options.input_pixel, options.output_pixel

		# Now that data contains information about each micrographs, let's window particles.
		window(data)
	else:
		print 'Error: Different number of coordinates and ctf information.'
		return 1


if __name__=='__main__':
	main()